# Hot directory list management
proc dirbrowser_do_menu {w dirlist x y} {
    global env
    upvar #0 $dirlist dl
    set cwd [dirbrowser_pwd]
    if [winfo exists $w.dirmenu] {
	$w.dirmenu delete 0 end
    } else {
	menu $w.dirmenu -tearoff 0
    }
    if {[lsearch -exact $dl $cwd]==-1} {
	$w.dirmenu add command -label "Add Current Directory" -command \
		[list lappend $dirlist $cwd]
    } else {
	$w.dirmenu add command -label "Add Current Directory" -state disabled
    }
    $w.dirmenu add command -label "Edit Directory Hotlist" -command \
	    [list dirbrowser_edit_dirlist $w $dirlist]
    $w.dirmenu add separator
    $w.dirmenu add command -label "Home Directory" -command \
	    [list dirbrowser_filterCommand $env(HOME)]
    foreach dir $dl {
	regexp {([^/]+|/)$} $dir shortdir
	$w.dirmenu add command -label $shortdir -command \
		[list dirbrowser_filterCommand $dir]
    }
    tk_popup $w.dirmenu $x $y
}

proc dirbrowser_edit_dirlist {w dirlist} {
    global dirbrowser
    upvar #0 $dirlist dl

    # Layout -----------------------------------------------------------------
    set p $w.dirlistedit
    toplevel $p -class DKFDirListEditor
    pack [frame $p.l] -side left -expand 1 -fill both
    pack [frame $p.r] -side left -fill both
    pack [listbox $p.l.l -yscrollcommand [list $p.l.s set] \
	    -selectmode single -exportselection 0] \
	    -side left -fill both -expand 1
    pack [scrollbar $p.l.s -command [list $p.l.l yview]] -side left -fill y

    pack [button $p.r.up -text "Move Item Up"	-command \
	    [list dirbrowser_edit_up     $p] -underline 10] -fill x
    pack [button $p.r.down -text "Move Item Down" -command \
	    [list dirbrowser_edit_down   $p] -underline 10] -fill x
    pack [button $p.r.ins -text "Insert Directory" -command \
	    [list dirbrowser_edit_insert $p] -underline 0] -fill x
    pack [button $p.r.mod -text "Modify Entry"	-command \
	    [list dirbrowser_edit_modify $p] -underline 0] -fill x
    pack [button $p.r.del -text "Delete Entry"	-command \
	    [list dirbrowser_edit_delete $p]] -fill x
    pack [button $p.r.hlp -text "Help"		-command \
	    [list dirbrowser_edit_help   $p]] -fill x -side bottom
    pack [button $p.r.can -text "Cancel"	-command \
	    [list dirbrowser_edit_cancel $p]] -fill x -side bottom
    pack [button $p.r.ok -text "O.K."		-command \
	    [list dirbrowser_edit_ok     $p]] -fill x -side bottom

    # Contents --------------------------------------------------------------
    foreach dir $dl {
	$p.l.l insert end $dir
    }
    if [llength $dl] {
	$p.l.l selection set 0
	$p.l.l see 0
    }

    # Bindings ---------------------------------------------------------------
    bind $p u [list $p.r.up invoke]
    bind $p d [list $p.r.down invoke]
    catch {
	bind $p <Insert> [list $p.r.ins invoke]
    }
    bind $p i [list $p.r.ins invoke]
    bind $p m [list $p.r.mod invoke]
    catch {
	bind $p <Delete> [list $p.r.del invoke]
    }
    catch {
	bind $p <BackSpace> [list $p.r.del invoke]
    }
    bind $p <Return> [list $p.r.ok invoke]
    bind $p <Escape> [list $p.r.can invoke]
    bind $p <F1> [list $p.r.hlp invoke]
    catch {
	bind $p <Help> [list $p.r.hlp invoke]
    }

    catch {
	# Don't know if these commands exist...
	dragdrop_destination $p.l.l accepts_drop {FILENAME STRING} \
		dirbrowser_edit_drop
	dragdrop_bind $p.l.l dirbrowser_edit_dragserver \
		"dirbrowser_edit_draggenerator $p.l.l \[$p.l.l get @%x,%y\]" 3
    }

    # toplevel configuration -------------------------------------------------
    wm withdraw $p
    wm title $p "Directory List Editor"
    if [llength [wm transient [winfo toplevel $w]]] {
	wm transient $p [wm transient [winfo toplevel $w]]
    } else {
	wm transient $p [winfo toplevel $w]
    }
    if [llength [wm group [winfo toplevel $w]]] {
	wm group $p [wm group [winfo toplevel $w]]
    }
    wm resizable $p 0 0
    catch {
	regexp {([-+][0-9]+)([-+][0-9]+)} \
		[wm geometry [winfo toplevel $w]] geom geomX geomY
	wm geometry $p +[expr 30 $geomX]+[expr 30 $geomY]
    }
    wm deiconify $p

    # O.K, wait until done. --------------------------------------------------
    with_grab_focus $p $p.l.l "tkwait window $p"
    if $dirbrowser(editbutton) {
	set dl $dirbrowser(editcontents)
    }
}

;proc dirbrowser_edit_drop {w x y s t d} {
    if {$t == "FILENAME" && ![file isdirectory $d]} {
	set dir [file dirname $d]
    } else {
	set dir $d
    }
    $w insert end {}
    $w insert "@$x,$y" $dir
    $w delete end
}
;proc dirbrowser_edit_dragserver {w t} {
    global dragdrop

    switch $t {
	STRING -
	FILENAME {
	    return $dragdrop(supplies,$w)
	}
	default {
	    return -code error "Wrong type code: $t instead of STRING"
	}
    }
}
;proc dirbrowser_edit_draggenerator {w s} {
    global dragdrop

    set dragdrop(supplies,$w) $s
    return {FILENAME STRING}
}

;proc dirbrowser_edit_entry {w default} {
    global dirbrowser
    set dirbrowser(dirlisteditentry) $default

    toplevel $w.entry -class DKFDirListEditorEntry
    pack [entry $w.entry.e -textvariable dirbrowser(dirlisteditentry)] -fill x
    pack [button $w.entry.ok -text O.K. -command \
	    [list dirbrowser_edit_entry_but $w.entry 0]] \
	    -expand 1 -side left -fill x
    pack [button $w.entry.can -text Cancel -command \
	    [list dirbrowser_edit_entry_but $w.entry 1]] \
	    -expand 1 -side left -fill x

    wm withdraw $w.entry
    if [llength [wm transient [winfo toplevel $w]]] {
	wm transient $w.entry [wm transient [winfo toplevel $w]]
    } else {
	wm transient $w.entry [winfo toplevel $w]
    }
    if [llength [wm group [winfo toplevel $w]]] {
	wm group $w.entry [wm group [winfo toplevel $w]]
    }
    wm title $w.entry "Entry box"
    wm resizable $w.entry 0 0
    catch {
	regexp {([-+][0-9]+)([-+][0-9]+)} \
		[wm geometry [winfo toplevel $w]] geom geomX geomY
	wm geometry $w.entry +[expr 30 $geomX]+[expr 30 $geomY]
    }
    wm deiconify $w.entry

    bind $w.entry <Return> [list $w.entry.ok invoke]
    bind $w.entry <Escape> [list $w.entry.can invoke]

    with_grab_focus $w.entry $w.entry.e "tkwait window $w.entry"
    if $dirbrowser(dirlisteditentrybutton) {
	return -code error "Cancelled"
    }
    return $dirbrowser(dirlisteditentry)
}
;proc dirbrowser_edit_entry_but {w f} {
    global dirbrowser
    set dirbrowser(dirlisteditentrybutton) $f
    destroy $w
}

;proc dirbrowser_edit_up {w} {
    set sel [$w.l.l curselection]
    if {[llength $sel] && $sel>0} {
	set el [$w.l.l get $sel]
	$w.l.l delete $sel
	incr sel -1
	$w.l.l insert $sel $el
	$w.l.l selection set $sel
	$w.l.l activate $sel
	$w.l.l see $sel
    }
}
;proc dirbrowser_edit_down {w} {
    set sel [$w.l.l curselection]
    if {[llength $sel] && $sel<[$w.l.l index end]} {
	set el [$w.l.l get $sel]
	$w.l.l delete $sel
	incr sel
	$w.l.l insert $sel $el
	$w.l.l selection set $sel
	$w.l.l activate $sel
	$w.l.l see $sel
    }
}
;proc dirbrowser_edit_insert {w} {
    set sel [$w.l.l curselection]
    if [llength $sel] {
	incr sel
    } else {
	set sel 0
    }
    catch {
	set newel [dirbrowser_edit_entry $w.l.l [dirbrowser_pwd]]
	$w.l.l insert $sel $newel
	$w.l.l selection clear 0 end
	$w.l.l selection set $sel
	$w.l.l activate $sel
	$w.l.l see $sel
    }
}
;proc dirbrowser_edit_modify {w} {
    set sel [$w.l.l curselection]
    if [llength $sel] {
	catch {
	    set newel [dirbrowser_edit_entry $w.l.l [$w.l.l get $sel]]
	    $w.l.l delete $sel
	    $w.l.l insert $sel $newel
	    $w.l.l selection set $sel
	    $w.l.l activate $sel
	    $w.l.l see $sel
	}
    }
}
;proc dirbrowser_edit_delete {w} {
    set sel [$w.l.l curselection]
    if [llength $sel] {
	set last [$w.l.l index end]
	$w.l.l delete $sel
	if {$sel==$last} {
	    incr sel -1
	}
	$w.l.l selection set $sel
	$w.l.l activate $sel
	$w.l.l see $sel
    }
}
;proc dirbrowser_edit_ok {w} {
    global dirbrowser
    set dirbrowser(editbutton) 1
    set dirbrowser(editcontents) [$w.l.l get 0 end]
    destroy $w
}
;proc dirbrowser_edit_cancel {w} {
    global dirbrowser
    set dirbrowser(editbutton) 0
    destroy $w
}
;proc dirbrowser_edit_help {w} {
    tk_dialog $w.help "Directory List Editor Help" "Use the Up and Down\
	    buttons to move the currently selected directory through the\
	    list. Insert will create a new entry (below the current\
	    entry), Modify will change the current entry and Delete will\
	    remove the current entry." info 0 {O.K.}
}
